package jsonx

import (
	"bytes"
	"encoding/json"
	"github.com/hsiafan/glow/stringx"
)

// Marshal encode value to json string, without html escape
func Marshal(v interface{}) (string, error) {
	var buf bytes.Buffer
	enc := json.NewEncoder(&buf)
	enc.SetEscapeHTML(false)
	err := enc.Encode(v)
	if err != nil {
		return "", err
	}
	data := buf.Bytes()
	l := len(data)
	if l > 0 && data[l-1] == '\n' {
		data = data[:l-1]
	}
	return stringx.FromBytes(data), err
}

// MustMarshal encode value to json string, without html escape. If encode failed, panics
func MustMarshal(v interface{}) string {
	jsonStr, err := Marshal(v)
	if err != nil {
		panic(err)
	}
	return jsonStr
}

// MarshalIndent is like Marshal but applies Indent to format the output.
// Each JSON element in the output will begin on a new line beginning with prefix
// followed by one or more copies of indent according to the indentation nesting.
func MarshalIndent(v interface{}, indent string) (string, error) {
	var buf bytes.Buffer
	enc := json.NewEncoder(&buf)
	enc.SetEscapeHTML(false)
	enc.SetIndent("", indent)
	err := enc.Encode(v)
	if err != nil {
		return "", err
	}
	data := buf.Bytes()
	l := len(data)
	if l > 0 && data[l-1] == '\n' {
		data = data[:l-1]
	}
	return stringx.FromBytes(data), err
}

// MustMarshalIndent encode value to json with ident. If error occurred, panics.
func MustMarshalIndent(v interface{}, indent string) string {
	jsonStr, err := MarshalIndent(v, indent)
	if err != nil {
		panic(err)
	}
	return jsonStr
}

// UnMarshal parse json encoded string to value
func UnMarshal(jsonStr string, v interface{}) error {
	return json.Unmarshal(stringx.ToBytes(jsonStr), v)
}

// MustUnMarshal parse json encoded string to value. If error occurred, panics
func MustUnMarshal(jsonStr string, v interface{}) {
	if err := UnMarshal(jsonStr, v); err != nil {
		panic(err)
	}
}
