package must

import (
	"github.com/hsiafan/glow/errorx"
	"io"
	"os"
)

// Run is used for check func with an error, and no return value.
// If err is not nil, panics with the error.
func Run(err error) {
	if err != nil {
		panic(err)
	}
}

// RunWithMessage is used for check func with an error, and no return value.
// If err is not nil, panics with a new error wrap the origin err and message.
func RunWithMessage(err error, message string) {
	if err != nil {
		panic(errorx.Wrap(message, err))
	}
}

// Call is used for check func returns an interface{} value and an error.
// If err is not nil, panics with the error.
func Call(value interface{}, err error) interface{} {
	if err != nil {
		panic(err)
	}
	return value
}

// CallBytes is used for check func returns a []byte value and an error.
// If err is not nil, panics with the error.
func CallBytes(value []byte, err error) []byte {
	if err != nil {
		panic(err)
	}
	return value
}

// CallBool is used for check func returns a bool value and an error.
// If err is not nil, panics with the error.
func CallBool(value bool, err error) bool {
	if err != nil {
		panic(err)
	}
	return value
}

// CallInt is used for check func returns an int value and an error.
// If err is not nil, panics with the error.
func CallInt(value int, err error) int {
	if err != nil {
		panic(err)
	}
	return value
}

// CallInt64 is used for check func returns an int64 value and an error.
// If err is not nil, panics with the error.
func CallInt64(value int64, err error) int64 {
	if err != nil {
		panic(err)
	}
	return value
}

// CallUInt is used for check func returns an uint value and an error.
// If err is not nil, panics with the error.
func CallUInt(value uint, err error) uint {
	if err != nil {
		panic(err)
	}
	return value
}

// CallUInt64 is used for check func returns an uint64 value and an error.
// If err is not nil, panics with the error.
func CallUInt64(value uint64, err error) uint64 {
	if err != nil {
		panic(err)
	}
	return value
}

// CallString is used for check func returns a string value and an error.
// If err is not nil, panics with the error.
func CallString(value string, err error) string {
	if err != nil {
		panic(err)
	}
	return value
}

// CallReader is used for check func returns an io.Reader value and an error.
// If err is not nil, panics with the error.
func CallReader(value io.Reader, err error) io.Reader {
	if err != nil {
		panic(err)
	}
	return value
}

// CallReadCloser is used for check func returns an io.ReadCloser value and an error.
// If err is not nil, panics with the error.
func CallReadCloser(value io.ReadCloser, err error) io.ReadCloser {
	if err != nil {
		panic(err)
	}
	return value
}

// CallWriter is used for check func returns an io.Writer value and an error.
// If err is not nil, panics with the error.
func CallWriter(value io.Writer, err error) io.Writer {
	if err != nil {
		panic(err)
	}
	return value
}

// CallWriteCloser is used for check func returns an io.WriteCloser value and an error.
// If err is not nil, panics with the error.
func CallWriteCloser(value io.WriteCloser, err error) io.WriteCloser {
	if err != nil {
		panic(err)
	}
	return value
}

// CallFile is used for check func returns an *os.File value and an error.
// If err is not nil, panics with the error.
func CallFile(value *os.File, err error) *os.File {
	if err != nil {
		panic(err)
	}
	return value
}

// CallFileInfo is used for check func returns an *os.FileInfo value and an error.
// If err is not nil, panics with the error.
func CallFileInfo(value *os.FileInfo, err error) *os.FileInfo {
	if err != nil {
		panic(err)
	}
	return value
}

//
//// Call value := must.Call(test())
//func Call[T interface{}](result T, err error) T {
//	if err != nil {
//		panic(err)
//	}
//	return result
//}
